import { CrawlerConfig, CrawlerDetectionResult, HeadersLike } from './types.js';
import { AI_CRAWLER_PATTERNS, SEO_CRAWLER_PATTERNS, matchesPatterns, detectCrawlerType } from './crawlers.js';

/**
 * Default configuration for crawler blocking
 */
export const DEFAULT_CONFIG: Required<CrawlerConfig> = {
  blockAI: true,
  blockSEO: false,
  message: 'Access denied',
  statusCode: 403,
  customBlocked: [],
  whitelist: [],
  headers: {},
  debug: false,
};

/**
 * Merge user config with default config
 */
export function mergeConfig(userConfig: CrawlerConfig = {}): Required<CrawlerConfig> {
  return {
    ...DEFAULT_CONFIG,
    ...userConfig,
    customBlocked: [...DEFAULT_CONFIG.customBlocked, ...(userConfig.customBlocked || [])],
    whitelist: [...DEFAULT_CONFIG.whitelist, ...(userConfig.whitelist || [])],
    headers: { ...DEFAULT_CONFIG.headers, ...(userConfig.headers || {}) },
  };
}

/**
 * Log debug information if debug mode is enabled
 */
function debug(config: Required<CrawlerConfig>, message: string): void {
  if (config.debug) {
    // eslint-disable-next-line no-console
    console.log(`[crawl-me-not] ${message}`);
  }
}

/**
 * Check if a user agent should be blocked based on the configuration
 */
export function shouldBlockCrawler(userAgent: string, config: CrawlerConfig = {}): CrawlerDetectionResult {
  const mergedConfig = mergeConfig(config);
  
  // Default result
  const result: CrawlerDetectionResult = {
    isBlocked: false,
    crawlerType: null,
    userAgent,
  };

  debug(mergedConfig, `Checking user agent: ${userAgent}`);

  // Check whitelist first (takes precedence)
  const whitelistMatch = matchesPatterns(userAgent, mergedConfig.whitelist);
  if (whitelistMatch.match) {
    debug(mergedConfig, `User agent whitelisted by pattern: ${String(whitelistMatch.pattern)}`);
    return result;
  }

  // Check custom blocked patterns
  const customMatch = matchesPatterns(userAgent, mergedConfig.customBlocked);
  if (customMatch.match && customMatch.pattern) {
    debug(mergedConfig, `User agent blocked by custom pattern: ${String(customMatch.pattern)}`);
    return {
      ...result,
      isBlocked: true,
      crawlerType: 'custom',
      matchedPattern: customMatch.pattern,
    };
  }

  // Detect crawler type
  const crawlerType = detectCrawlerType(userAgent);
  
  if (crawlerType === 'ai' && mergedConfig.blockAI) {
    const aiMatch = matchesPatterns(userAgent, AI_CRAWLER_PATTERNS);
    if (aiMatch.pattern) {
      debug(mergedConfig, `AI crawler blocked: ${String(aiMatch.pattern)}`);
      return {
        ...result,
        isBlocked: true,
        crawlerType: 'ai',
        matchedPattern: aiMatch.pattern,
      };
    }
  }

  if (crawlerType === 'seo' && mergedConfig.blockSEO) {
    const seoMatch = matchesPatterns(userAgent, SEO_CRAWLER_PATTERNS);
    if (seoMatch.pattern) {
      debug(mergedConfig, `SEO crawler blocked: ${String(seoMatch.pattern)}`);
      return {
        ...result,
        isBlocked: true,
        crawlerType: 'seo',
        matchedPattern: seoMatch.pattern,
      };
    }
  }

  debug(mergedConfig, 'User agent allowed');
  return {
    ...result,
    crawlerType,
  };
}

/**
 * Extract user agent from various header formats
 */
export function extractUserAgent(headers: HeadersLike): string {
  // Handle Headers-like object (Web API, SvelteKit, etc.)
  if (typeof (headers as { get(name: string): string | null }).get === 'function') {
    const headersObj = headers as { get(name: string): string | null };
    return headersObj.get('user-agent') || headersObj.get('User-Agent') || '';
  }
  
  // Handle regular object (Express, Node.js, etc.)
  const headersObj = headers as Record<string, string | string[] | undefined>;
  const userAgent = headersObj['user-agent'] || headersObj['User-Agent'];
  
  if (Array.isArray(userAgent)) {
    return userAgent[0] || '';
  }
  
  return userAgent || '';
} 