/**
 * Known AI crawler user agent patterns
 * These are patterns for bots that are primarily used for AI training, data collection, or content scraping
 */
export const AI_CRAWLER_PATTERNS: (string | RegExp)[] = [
  // OpenAI
  /GPTBot/i,
  /ChatGPT-User/i,
  
  // Google AI
  /Google-Extended/i,
  /GoogleOther/i,
  
  // Anthropic
  /Claude-Web/i,
  /ClaudeBot/i,
  
  // Meta/Facebook AI
  /FacebookBot/i,
  /Meta-ExternalAgent/i,
  
  // Bytedance/TikTok
  /Bytespider/i,
  /ByteDance/i,
  
  // Common AI/ML crawlers
  /CCBot/i,
  /anthropic-ai/i,
  /PerplexityBot/i,
  /YouBot/i,
  /ChatGPT/i,
  /GPT/i,
  /OpenAI/i,
  /AI2Bot/i,
  /cohere-ai/i,
  
  // Academic/Research crawlers often used for AI training
  /ArchiveBot/i,
  /Internet Archive/i,
  /archive\.org/i,
  
  // Content scrapers and data collectors
  /DataForSeoBot/i,
  /SemrushBot/i,
  /AhrefsBot/i,
  /MJ12bot/i,
  /DotBot/i,
  /CommonCrawl/i,
  /webzio/i,
  /Scrapy/i,
  /scrapy/i,
  /python-requests/i,
  /python-urllib/i,
  /curl/i,
  /wget/i,
  /HTTPie/i,
  /Postman/i,
  /Insomnia/i,
  
  // Generic AI/bot patterns
  /bot.*ai/i,
  /ai.*bot/i,
  /crawler/i,
  /scraper/i,
  /spider/i,
];

/**
 * Known SEO crawler user agent patterns
 * These are legitimate crawlers used for SEO analysis and website monitoring
 */
export const SEO_CRAWLER_PATTERNS: (string | RegExp)[] = [
  // Google SEO tools
  /Googlebot/i,
  /Google-Site-Verification/i,
  /Google-InspectionTool/i,
  
  // Bing
  /Bingbot/i,
  /BingPreview/i,
  /msnbot/i,
  
  // Yandex
  /YandexBot/i,
  /YandexImages/i,
  /YandexMetrika/i,
  
  // Baidu
  /Baiduspider/i,
  /Baidu/i,
  
  // DuckDuckGo
  /DuckDuckBot/i,
  /DuckDuckGo/i,
  
  // SEO tools
  /AhrefsBot/i,
  /SemrushBot/i,
  /MJ12bot/i,
  /DotBot/i,
  /MegaIndex/i,
  /BacklinkCrawler/i,
  /SEOkicks/i,
  /sistrix/i,
  /BLEXBot/i,
  
  // Social media crawlers
  /facebookexternalhit/i,
  /Twitterbot/i,
  /LinkedInBot/i,
  /WhatsApp/i,
  /TelegramBot/i,
  /SkypeUriPreview/i,
  /Slackbot/i,
  /Discordbot/i,
  
  // Other search engines
  /Yahoo/i,
  /Slurp/i,
  /Ask Jeeves/i,
  /Teoma/i,
  /ia_archiver/i,
  /Wayback/i,
];

/**
 * Check if a user agent matches any pattern in the given list
 */
export function matchesPatterns(userAgent: string, patterns: (string | RegExp)[]): { match: boolean; pattern?: string | RegExp } {
  const lowerUserAgent = userAgent.toLowerCase();
  
  for (const pattern of patterns) {
    if (pattern instanceof RegExp) {
      if (pattern.test(userAgent)) {
        return { match: true, pattern };
      }
    } else if (lowerUserAgent.includes(pattern.toLowerCase())) {
      return { match: true, pattern };
    }
  }
  
  return { match: false };
}

/**
 * Detect the type of crawler based on user agent
 */
export function detectCrawlerType(userAgent: string): 'ai' | 'seo' | null {
  const aiMatch = matchesPatterns(userAgent, AI_CRAWLER_PATTERNS);
  if (aiMatch.match) {
    return 'ai';
  }
  
  const seoMatch = matchesPatterns(userAgent, SEO_CRAWLER_PATTERNS);
  if (seoMatch.match) {
    return 'seo';
  }
  
  return null;
} 